// CadGameEx9.java

import ch.aplu.jcardgame.*;
import ch.aplu.jgamegrid.*;
import java.awt.*;

public class CardGameEx9 extends CardGame implements GGMouseTouchListener
{
  public enum Suit
  {
    KREUZ, KARO, HERZ, PIK
  }

  public enum Rank
  {
    ASS, KOENIG, DAME, BUBE, ZEHN, NEUN, ACHT, SIEBEN, SECHS
  }

  private Deck deck = new Deck(Suit.values(), Rank.values(), "cover");
  private Point hotspot = new Point(0, 0);
  private final Location talonLocation = new Location(300, 400);
  private Hand hand = new Hand(deck);
  private final Location handLocation = new Location(300, 120);
  private Card movingCard;

  public CardGameEx9()
  {
    super(600, 600);
    setStatusText("Drag cads from the stack to the hand");
    Hand[] hands = deck.dealingOut(1, 3);
    hand = hands[0];
    hands[0].setView(this, new RowLayout(handLocation,350));
    hands[0].draw();
    hand.setTouchEnabled(true);

    final Hand talon = hands[1];
    talon.setView(this, new StackLayout(talonLocation));
    talon.reverse(true);
    talon.addCardListener(new CardAdapter()
    {
      public void leftPressed(Card card)
      {
        talon.remove(card, true);
        movingCard = card.clone();
        add(movingCard);
      }
    });
    talon.setTouchEnabled(true);
  }

  private void add(Card card)
  {
    CardActor cardActor = card.getCardActor();
    addActor(cardActor, new Location(talonLocation));
    cardActor.addMouseTouchListener(this,
      GGMouse.lPress | GGMouse.lDrag | GGMouse.lRelease, false);
  }

  public void mouseTouched(Actor actor, GGMouse mouse, Point spot)
  {
    switch (mouse.getEvent())
    {
      case GGMouse.lPress:
        hotspot = spot;
        break;

      case GGMouse.lDrag:
        actor.setLocation(
          new Location(mouse.getX() - hotspot.x, mouse.getY() - hotspot.y));
        if (actor.getLocation().getDistanceTo(handLocation) < 100)
          putInHand(actor);
        break;

      case GGMouse.lRelease:
        putInHand(actor);
        break;
    }
  }

  private void putInHand(Actor actor)
  {
    actor.removeSelf();
    Card card = ((CardActor)actor).getCard();
    hand.insert(card, true);
    hand.sort(Hand.SortType.SUITPRIORITY, true);
  }

  public static void main(String[] args)
  {
    new CardGameEx9();
  }
}